# -*- coding: utf-8 -*-
# This file is part of ekd - Music and sound module
# Copyright (C) 2007-2009  Olivier Ponchaut <opvg@edpnet.be>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os
import subprocess
from PyQt4.QtCore import QObject, SIGNAL, QProcess, QByteArray, QString, QThread, QFile
from PyQt4.QtGui import QWidget, QHBoxLayout, QGridLayout, QLabel, QComboBox, QCheckBox, QGroupBox, QSpacerItem, QSizePolicy
from gui_modules_common.EkdWidgets import EkdProgress
from moteur_modules_common.EkdConfig import EkdConfig
from moteur_modules_common.EkdProcess import EkdProcess

###############
from moteur_modules_common.EkdCompatiblePython2Et3 import EkdPrint
###############


if os.name in ['posix', 'mac']:
	coding = "utf-8"
else :
	coding = "cp1252"


class reglageExpert(QWidget) :
	"""Class pour créer les widgets spécifiques pour les réglages plus fin sur la qualité d'encodage, sur le débit binaire des fichiers, ... """
	def __init__(self, listeFormat, comboFormat, parent=None) :
		super(reglageExpert, self).__init__(parent)
		self.listeFormat = listeFormat
		self.comboFormat = comboFormat
		self.format = self.comboFormat.currentIndex()
		self.parent = parent
		layout = QHBoxLayout(self)
		self.textQualite = QLabel(self)
		self.textQualite.hide()
		self.textRate = QLabel(self)
		self.textRate.hide()
		self.expert = QCheckBox(_(u"Réglage expert : "), self)
		self.qualite = QComboBox(self)
		self.rate = QComboBox(self)
		Glayout = QGridLayout()
		Glayout.addWidget(self.textQualite,0,0)
		Glayout.addWidget(self.qualite,0,1)
		Glayout.addWidget(self.textRate,1,0)
		Glayout.addWidget(self.rate,1,1)
		layout.addWidget(self.expert)
		layout.addLayout(Glayout)
		layout.addStretch()
		self.qualite.addItem(QString(u"64k"))
		self.qualite.addItem(QString(u"128k"))
		self.qualite.addItem(QString(u"192k"))
		self.qualite.addItem(QString(u"224k"))
		self.qualite.addItem(QString(u"384k"))
		self.qualite.hide()
		self.rate.addItem(QString(u"22050 Hz"))
		self.rate.addItem(QString(u"44100 Hz"))
		self.rate.addItem(QString(u"48000 Hz"))
		self.rate.hide()
		self.changeFormat(self.comboFormat.currentIndex())
		self.connect(self.expert, SIGNAL("stateChanged(int)"), self.setExpert)
		self.connect(self.comboFormat, SIGNAL("currentIndexChanged(int)"), self.changeFormat)
		self.connect(self.qualite, SIGNAL("currentIndexChanged(int)"), self.changeQualite)

	def setExpert(self, state) :
		if state :
			self.qualite.show()
			self.textQualite.show()
			self.rate.show()
			self.textRate.show()
		else :
			self.qualite.hide()
			self.textQualite.hide()
			self.rate.hide()
			self.textRate.hide()

	def changeQualite(self,qualite) :
		self.changeFormat(self.comboFormat.currentIndex())

	def changeFormat(self, format) :
		if len(self.listeFormat) == 0 :
			return
		else :
			if self.listeFormat[format][0] == "ogg" :
				self.acodec = u"-acodec libvorbis"
				self.qualite.setEnabled(True)
			elif self.listeFormat[format][0] == "wma" :
				self.acodec = u"-acodec wmav2"
				self.qualite.setEnabled(True)
			elif self.listeFormat[format][0] == "ac3" :
				self.acodec = u"-acodec ac3"
				self.qualite.setEnabled(True)
			elif self.listeFormat[format][0] == "aac" :
				self.acodec = u"-acodec aac -strict experimental"
				self.qualite.setEnabled(True)
			elif self.listeFormat[format][0] == "flac" :
				self.acodec = u"-acodec flac"
				self.qualite.setEnabled(True)
			elif self.listeFormat[format][0] == "wav" :
				self.acodec = u""
				self.qualite.setEnabled(False)
			else :
				self.acodec = u""
				self.qualite.setEnabled(True)

	def getExpertState(self) :
		return self.expert.isChecked()

	def getC(self) :
		rate = unicode(self.rate.currentText())[:-3]
		return [self.acodec, self.qualite.currentText(), rate]

	def saveConfig(self, idsection="") :
		if idsection == "" :
			return 
		EkdConfig.set(idsection, u'format', unicode(self.comboFormat.currentIndex()))
		EkdConfig.set(idsection, u'expert', unicode(int(self.getExpertState())))
		EkdConfig.set(idsection, u'qualite', unicode(self.qualite.currentIndex()))
		EkdConfig.set(idsection, u'rate', unicode(self.rate.currentIndex()))


	def loadConfig(self, idsection="") :
		if idsection == "" :
			return 
		self.comboFormat.setCurrentIndex(int(EkdConfig.get(idsection, 'format')))
		self.expert.setChecked(int(EkdConfig.get(idsection, 'expert')))
		self.qualite.setCurrentIndex(int(EkdConfig.get(idsection, 'qualite')))
		self.rate.setCurrentIndex(int(EkdConfig.get(idsection, 'rate')))


class checkSoxEncoding(QObject) :
	"""checkSoxEncoding est une class pour vérifier les possibilités de Sox pour l'encodage du format mp2/mp3. Si ce n'est pas le cas, retourne 0 -> adaptation de l'interface pour supprimer ces possibilités"""
	def __init__(self, listeformat, parent=None) :
		super(checkSoxEncoding, self).__init__(parent)
		#self.hide()
		self.listeFormat = listeformat
		self.parent = parent
		self.index = 0
		self.checkListeFormat = []

	def run(self) :
		self.soxP = QProcess(self.parent)
		self.connect(self.soxP, SIGNAL('finished(int)'), self.finProcess)
		self.write = 0
		commande = u"sox -n -t %s \"%s\" trim 00:00:00.000 00:00:01.000" % (self.listeFormat[self.index], EkdConfig.temp+os.sep+u"checkSoxEssai."+self.listeFormat[self.index])
		### Debug
		#print commande
		EkdPrint(unicode(commande))
		self.soxP.setProcessChannelMode(QProcess.MergedChannels)
		self.soxP.start(commande)

	def finProcess(self,statut) :
		# Test si la commande a été exécutée correctement
		#print "statut de sortie : %d" % (statut)
		EkdPrint(u"statut de sortie: %s" % (statut))
		if not statut :
			# Si OK -> Ajout de l'extension dans la liste des formats supportés par Sox en encodage
			self.checkListeFormat.append((self.listeFormat[self.index],True))
			os.remove(EkdConfig.temp+os.sep+u"checkSoxEssai."+self.listeFormat[self.index])
		else :
			# Sinon, déclaré le format comme non utilisable et donc pas présent dans la liste des formats supportés
			self.checkListeFormat.append((self.listeFormat[self.index],False))
		# Incrémentation du compteur pour explorer tous les formats à tester
		self.index += 1
		### Debug
		#print "index = %d" % (self.index)
		EkdPrint(u"index = " + unicode(self.index))
		#print "longueur liste = %d" % (len(self.listeFormat))
		EkdPrint(u"longueur liste = " + unicode(len(self.listeFormat)))
		# Si l'index est égal au nombre de format à tester, la class émet le signal de fin de vérification + liste des formats avec info sur le support
		if self.index == len(self.listeFormat) :
			### Debug : print self.checkListeFormat
			self.emit(SIGNAL("checkSox"), self.checkListeFormat)
		else :
			self.run()

class choixSonSortieWidget(QGroupBox) :
    """Class pour faire le widget standard de sélection du format de sortie son + réglages experts"""
    def __init__(self, parent=None) :
      super(choixSonSortieWidget, self).__init__(parent)
      self.setObjectName("groupBox")
      self.horizontalLayout = QHBoxLayout(self)
      self.horizontalLayout.setObjectName("horizontalLayout")
      self.label = QLabel(self)
      self.label.setObjectName("label")
      self.horizontalLayout.addWidget(self.label)
      self.formatSound = QComboBox(self)
      self.Format = [("wav","wav"), ("mp3","mp3"), ("mp2","mp2"), ("ogg","ogg"), ("flac","flac") ,("wma","wma"), ("m4a","m4a"), ("ac3","ac3"), ("aac","aac")]
      for x in self.Format :
        self.formatSound.addItem(QString(x[0]))
      self.formatSound.setObjectName("formatSound")
      self.horizontalLayout.addWidget(self.formatSound)
      spacerItem = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
      self.horizontalLayout.addItem(spacerItem)
      self.reglageExp = reglageExpert(self.Format, self.formatSound)
      self.horizontalLayout.addWidget(self.reglageExp)
      self.setTitle(_(u"Réglage du format de sortie"))
      self.label.setText(_(u"Format du fichier de sortie :"))
      self.formatSound.setToolTip(_(u"Format du fichier son de sortie"))

    def getFileExt(self) :
      return self.Format[self.formatSound.currentIndex()][1]

### Remplacé par SoundProcess et SoundEngine

### progression avec la class de traitement.
#class SoxProcess(EkdProgress) :
	#"""Class pour effectuer les traitements audio avec Sox et afficher la progression dans
		#une boîte de dialogue standard.
		#La class nécessite le ou les url des fichiers en entrée à traiter, l'url du fichier de sortie et les options à passer à SoX.
		#Le process génère 2 signaux (Données disponibles pour lire + données, fin de process + statut de sortie du process). Les critères pour définir l'avancement du process ainsi que les informations à afficher à l'utilisateurs doivent être définie au moyen des critères "critereFile" et "critereProgress"
	#"""
	#def __init__(self, cheminAudioSource, cheminAudioSorti, nfile, globalOptions=u"", outputFileOptions=u"", effects=u"", parent=None) :
		#super(SoxProcess, self).__init__(parent, 100)
		#self.processAudio = soxEngine(cheminAudioSource, cheminAudioSorti, nfile, globalOptions, outputFileOptions, effects, parent)


	#def retranslateUi(self, showprogress):
		#showprogress.setWindowTitle(_(u"Affichage de la progression"))
		#self.barProgress.setToolTip(_(u"Progression de l\'encodage"))
		#self.infoText.setToolTip(_(u"Messages de SoX"))
		#self.fermer.setToolTip(_(u"Fermer la fenêtre"))
		#self.fermer.setText(_(u"Fermer"))
		#self.fermer.setShortcut(_(u"Alt+S"))

	#def setSignal(self, critereFile, critereProgress) :
		#self.processAudio.critereFile = critereFile
		#self.processAudio.critereProgress = critereProgress

	#def run(self) :
		#self.connect(self.processAudio, SIGNAL("progress(int)"), self.setProgress)
		#self.connect(self.processAudio, SIGNAL("endProcess(QString)"), self.endAction)
		#self.connect(self.processAudio, SIGNAL("DisplayText"), self.addText)
		#self.processAudio.start()

	#def setProgress(self, prg) :
		#self.barProgress.setValue(prg)

	#def endAction(self, msg) :
		#self.barProgress.setValue(100)
		#self.fermer.setEnabled(True)
		#self.emit(SIGNAL("endProcess"), msg)

#class soxEngine(QThread) :

	#def __init__(self, cheminAudioSource, cheminAudioSorti, nfile, globalOptions=u"", outputFileOptions=u"", effects=u"", parent=None) :
		#super(soxEngine, self).__init__(parent)
		#self.cheminAudioSource = unicode(cheminAudioSource)
		#self.cheminAudioSorti = unicode(cheminAudioSorti)
		#self.effects = unicode(effects)
		#self.globalOptions = unicode(globalOptions)
		#self.outputFileOptions = unicode(outputFileOptions)
		#self.critereFile = u""
		#self.critereProgress = u""
		#self.nfile = nfile
		#self.avancement = 0
		#self.avancementFile = 0
		#self.fileT = -1


	#def run(self):
		#commande = u"sox -S "+self.globalOptions+u" "+self.cheminAudioSource+u" "+self.outputFileOptions+u" \""+self.cheminAudioSorti+u"\" "+self.effects
		#### Debug
		##print commande
		#try :
			##self.mesg = subprocess.Popen(commande.encode(EkdConfig.coding), universal_newlines=True, bufsize=0, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, preexec_fn=os.setsid)
			#self.mesg = EkdProcess(commande.encode(EkdConfig.coding), outputerr=subprocess.STDOUT)
		#except Exception, e:
			##print u"Erreur dans le lancement de %s\n(%s)" % (commande, e)
			#EkdPrint(u"Erreur dans le lancement de " + str(commande) + "\n(" + e + ")")
			#raise
		#self.retCode = None
		#while self.retCode == None :
			#self.progression()
			#self.retCode = self.mesg.poll()
			##self.usleep(10)
		#self.finEncodage(self.retCode)

	#def progression(self):
		#"""
		#Recupération de la progression du process en cours
		#"""
		#tampon = self.mesg.stdout.readline()
		#### Debug
		##print tampon
		#progre = 0
		## on récupère les lignes de Sox
		#if self.critereFile != u"" and tampon.find(str(self.critereFile))!= -1 :
			#self.emit(SIGNAL("DisplayText"),QString(tampon))
			#self.fileT += 1
			#avInst = float(self.fileT*100/self.nfile)
			#if self.avancementFile < avInst : 
				#self.avancementFile = avInst
				#self.emit(SIGNAL("progress(int)"),int(self.avancementFile))

		#if self.critereProgress != u"" and tampon.find(str(self.critereProgress))!= -1 :
			#try :
				#pr1 = tampon.split(":")
				#pr2 = pr1[1].split("%")
				#progre = int(float(pr2[0]))
			#except :
				#None
			#avInst = self.avancementFile + float(progre/self.nfile)
			#if self.avancement < avInst : 
				#self.avancement = avInst
				#self.emit(SIGNAL("progress(int)"),int(self.avancement))

		#elif self.critereProgress != u"" and tampon.find("Time:")!=-1 :
			#try :
				#pr1 = tampon.split("(")
				#pr2 = pr1[1].split("%")
				#progre = int(float(pr2[0]))
			#except :
				#None
			#avInst = self.avancementFile + float(progre/self.nfile)
			#if self.avancement < avInst : 
				#self.avancement = avInst
				#self.emit(SIGNAL("progress(int)"),int(self.avancement))

	#def finEncodage(self, statutDeSortie):
		#"""choses à faire à la fin de l'encodage audio"""
		#if statutDeSortie==1:
			##print "Crash de l'encodeur!"
			#EkdPrint(u"Crash de l'encodeur!")
			#self.emit(SIGNAL("DisplayText"),u"Crash de l'encodeur !")
			#self.fermer.setEnabled(True)
			#return

		#else :
			#self.emit(SIGNAL("endProcess(QString)"), self.cheminAudioSorti)
			##print self.cheminAudioSorti
			#EkdPrint(str(self.cheminAudioSorti))
			#self.emit(SIGNAL("DisplayText"),u"Encodage terminé avec succès")
#

class SoundProcess(EkdProgress) :
	"""Class pour effectuer les traitements audio avec Sox et afficher la progression dans
		une boîte de dialogue standard.
		La class nécessite le ou les url des fichiers en entrée à traiter, l'url du fichier de sortie et les options à passer à SoX.
		Le process génère 2 signaux (Données disponibles pour lire + données, fin de process + statut de sortie du process). Les critères pour définir l'avancement du process ainsi que les informations à afficher à l'utilisateurs doivent être définie au moyen des critères "critereFile" et "critereProgress"
	"""
	def __init__(self, cheminAudioSource, cheminAudioSorti, acodec=u"", TstartV=0.0, ar=48000, ab=128000, vol=256, duration=-1, fadeIn=0.0, fadeOut=0.0, soxOpt=u"", parent=None) :
		super(SoundProcess, self).__init__(parent, 100)
		self.processAudio = soundEngine(cheminAudioSource, cheminAudioSorti, acodec, TstartV, ar, ab, vol, duration, fadeIn, fadeOut, soxOpt, parent)
		self.output = []


	def retranslateUi(self):
		self.setWindowTitle(_(u"Affichage de la progression"))
		self.barProgress.setToolTip(_(u"Progression de l\'encodage"))
		self.infoText.setToolTip(_(u"Messages de SoX"))
		self.fermer.setToolTip(_(u"Fermer la fenêtre"))
		self.fermer.setText(_(u"Fermer"))
		self.fermer.setShortcut(_(u"Alt+S"))

	def setSignal(self, critereFile, critereProgress) :
		self.processAudio.critereFile = critereFile
		self.processAudio.critereProgress = critereProgress

	def run(self) :
		self.connect(self.processAudio, SIGNAL("progress(int)"), self.setProgress)
		self.connect(self.processAudio, SIGNAL("endProcess(QString)"), self.endAction)
		self.connect(self.processAudio, SIGNAL("endProcessFinal"), self.endActionFinal)
		self.connect(self.processAudio, SIGNAL("DisplayText"), self.addText)
		self.processAudio.start()

	def setProgress(self, prg) :
		self.barProgress.setValue(prg)

	def endAction(self, msg) :
		self.output.append(unicode(msg))
		
	def endActionFinal(self) :
		self.barProgress.setValue(100)
		self.fermer.setEnabled(True)
		self.emit(SIGNAL("endProcess"), self.output)

class soundEngine(QThread) :

	def __init__(self, cheminAudioSource, cheminAudioSorti, acodec=u"", TstartV=0.0, ar=48000, ab=128000, vol=256, duration=-1, fadeIn=0.0, fadeOut=0.0, soxOpt=u"", parent=None) :
		super(soundEngine, self).__init__(parent)
		self.multi = False
		self.nfile = 1
		if type(cheminAudioSource) == list :
			self.multi = True
			self.nfile = len(cheminAudioSource)
			self.cheminAudioSource = cheminAudioSource
		else :
			self.cheminAudioSource = unicode(cheminAudioSource)
		self.cheminAudioSorti = unicode(cheminAudioSorti)
		self.acodec=acodec
		self.TstartV = TstartV
		self.ar=ar
		self.ab=ab
		self.vol=vol
		self.duration=duration
		self.fadeIn=fadeIn
		self.fadeOut=fadeOut
		self.avancement = 0
		self.avancementFile = 0
		self.fileT = -1
		self.soxOpt = soxOpt


	def run(self):
		i=0
		while i < self.nfile :
			if self.multi :
				sourceAudio = self.cheminAudioSource[i]
				audioOut = u"%s%03d%s" % (self.cheminAudioSorti[:-4],i+1,self.cheminAudioSorti[-4:])
			else :
				sourceAudio = self.cheminAudioSource
				audioOut = self.cheminAudioSorti
				
			commande = self.buildSound(urlFFmpeg=EkdConfig.urlFFmpeg, urlSox=EkdConfig.urlSox, urlIn=sourceAudio, urlOut=audioOut, TstartV=self.TstartV, ar=self.ar, ab=self.ab, vol=self.vol, duration=self.duration, fadeIn=self.fadeIn, fadeOut=self.fadeOut, soxOpt=self.soxOpt)
			try :
				self.mesg = EkdProcess(commande.encode(EkdConfig.coding), outputerr=subprocess.STDOUT)
			except Exception, e:
				EkdPrint(u"Erreur dans le lancement de " + unicode(commande) + "\n(" + e + ")")
				raise
			self.retCode = None
			while self.retCode == None :
				self.progression(i)
				self.retCode = self.mesg.poll()
				#self.usleep(10)
			self.finEncodage(self.retCode, i)
			i += 1
		self.finEncodageFinal()
			
	def progression(self, i):
		"""
		Recupération de la progression du process en cours
		"""
		tampon = self.mesg.stdout.readline()
		progre = 0
		self.emit(SIGNAL("DisplayText"),QString(tampon))
		progress = int(100.0*i/self.nfile)
		self.emit(SIGNAL("progress(int)"),progress)


	def finEncodageFinal(self) :
			self.emit(SIGNAL("DisplayText"),_(u"Encodage de tous les fichiers à traiter terminé"))
			self.emit(SIGNAL("endProcessFinal"))

	def finEncodage(self, statutDeSortie, i):
		"""choses à faire à la fin de l'encodage audio"""
		if self.multi :
			outputFile = u"%s%03d%s" % (self.cheminAudioSorti[:-4],i+1,self.cheminAudioSorti[-4:])
		else :
			outputFile = self.cheminAudioSorti
			
		if not QFile().exists(outputFile) :
			#print "Crash de l'encodeur!"
			EkdPrint(u"Crash de l'encodeur!")
			self.emit(SIGNAL("DisplayText"),_(u"Crash de l'encodeur !"))
			return

		else :
			#print self.cheminAudioSorti
			EkdPrint(unicode(outputFile))
			self.emit(SIGNAL("DisplayText"),_(u"Encodage terminé avec succès"))
			self.emit(SIGNAL("endProcess(QString)"), outputFile)
			return
			
	### Fonction encodage sox avec FFMPEG
	def buildSound(self, urlFFmpeg, urlSox, urlIn, urlOut, TstartV, ar=48000, ab=128000, vol=256, duration=-1, fadeIn=0.0, fadeOut=0.0, soxOpt=u"") :
		"""Function to compose sound with fade in, fade out, from movie sound ou sound file"""
		# suffix du fichier entrant
		suffixIn=os.path.splitext(urlIn)[1]
		if suffixIn == u".wav" or suffixIn == u".WAV" :
			wav = 1
		else :
			wav = 0

		if duration == -1 :
			fade = u""
			durFF = u""
		else :
			fade = u"fade h %f %f %f" % (fadeIn, duration, fadeOut)
			durFF =  u"-t %f" % (duration)

		if duration==-1 and fadeIn==0.0 and fadeOut==0.0 and soxOpt==u"" :
			command = u"%sffmpeg -y -i \"%s\" %s -ac 2 -ar %d -ab %s -ss %f \"%s\"" % ( \
										urlFFmpeg, \
										urlIn, \
										self.acodec, \
										int(ar), \
										ab, \
										TstartV, \
										urlOut)
		elif wav :
			command = u"%ssox \"%s\" -t wav -c 2 -r %d -b 16 --endian little -e signed-integer - trim %s %s %s | \"%sffmpeg\" -y -acodec pcm_s16le -ar %d -ac 2 -vol %d -f wav -i - %s -ar %d -ab %s \"%s\"" % ( \
										urlSox, \
										urlIn, \
										int(ar), \
										TstartV, \
										soxOpt, \
										fade, \
										urlFFmpeg, \
										int(ar), \
										vol, \
										self.acodec, \
										int(ar), \
										ab, \
										urlOut)
		else :
			command = u"%sffmpeg -i \"%s\" -vn -acodec pcm_s16le -ar %d -ac 2 %s -ss %f -vol %d -f s16le - | \"%ssox\" -t raw -c 2 -r %d -b 16 --endian little -e signed-integer - -t wav -c 2 -r %d -b 16 --endian little -e signed-integer - %s %s | \"%sffmpeg\" -y -acodec pcm_s16le -ar %d -ac 2 -f wav -i - %s -ab %s \"%s\"" % ( \
										urlFFmpeg, \
										urlIn, \
										int(ar), \
										durFF, \
										TstartV, \
										vol, \
										urlSox, \
										int(ar), \
										int(ar), \
										soxOpt, \
										fade, \
										urlFFmpeg, \
										int(ar), \
										self.acodec, \
										ab, \
										urlOut)
		EkdPrint(command)
		return command


		### Supprimé temporairement
"""		if self.critereProgress != u"" and tampon.find(str(self.critereProgress))!= -1 :
			try :
				pr1 = tampon.split(":")
				pr2 = pr1[1].split("%")
				progre = int(float(pr2[0]))
			except :
				None
			avInst = self.avancementFile + float(progre/self.nfile)
			if self.avancement < avInst :
				self.avancement = avInst
				self.emit(SIGNAL("progress(int)"),int(self.avancement))

		elif self.critereProgress != u"" and tampon.find("Time:")!=-1 :
			try :
				pr1 = tampon.split("(")
				pr2 = pr1[1].split("%")
				progre = int(float(pr2[0]))
			except :
				None
			avInst = self.avancementFile + float(progre/self.nfile)
			if self.avancement < avInst :
				self.avancement = avInst
				self.emit(SIGNAL("progress(int)"),int(self.avancement))
"""
