# -*- coding: utf-8 -*-

import os, string, subprocess
from PyQt4.QtCore import SIGNAL, SLOT, QThread

from moteur_modules_common.EkdProcess import EkdProcess
from moteur_modules_common.EkdConfig import EkdConfig
from moteur_modules_common.EkdTools import debug
from moteur_modules_common.EkdCompatiblePython2Et3 import EkdPrint

############################################################################################
# Merci à l'auteur de: http://snippets.prendreuncafe.com/snippets/tagged/pyqt/order_by/date
# pour son code ... et son aide
############################################################################################

class FfmpegAvchd(QThread):

	### Ajout de choixDesent (le 14/07/2011) pour le désentrelacement ##################
	def __init__(self, cheminVideoEntre, enregistrerSortie, choixDesent, codec_sortie, reso_largeur_sortie, reso_hauteur_sortie, nbreImgSec_sortie, qualite_sortie, spec_sortie_DNxHD, son_sortie_DNxHD, mode_sortie_DV, aspect_sortie_DV, son_sortie_General, cheminFFmpeg=None):

		QThread.__init__(self)
		
		self.cancel = False
		
		#=== Paramètres généraux ===#
		
		# Uniquement pour Linux et MacOSX
		if os.name in ['posix', 'mac']: self.cheminFFmpeg = "ffmpeg"
			
		# Uniquement pour windows
		elif os.name == 'nt': self.cheminFFmpeg = u"%sffmpeg.exe" % (EkdConfig.urlFFmpeg)

		# chemin(s) des vidéos chargées
		self.cheminVideoEntre = cheminVideoEntre
		# Chemin + nom de fichier en sortie
		self.enregistrerSortie = enregistrerSortie
		# Choix du désentrelacement (oui ou non)
		self.choixDesent = choixDesent
		# Sélection du codec en sortie (par l'utilisateur)
		self.codec_sortie = codec_sortie
		# Sélection de la largeur (vidéo) en sortie (par l'utilisateur)
		self.reso_largeur_sortie = reso_largeur_sortie
		# Sélection de la hauteur (vidéo) en sortie (par l'utilisateur)
		self.reso_hauteur_sortie = reso_hauteur_sortie
		# Sélection du nbr d'img/sec en sortie (par l'utilisateur)
		self.nbreImgSec_sortie = nbreImgSec_sortie
		# Sélection de la qualité (de la vidéo) en sortie (par l'utilisateur)
		self.qualite_sortie = qualite_sortie
		# Spécifications pour l'Avid DNxHD
		self.spec_sortie_DNxHD = spec_sortie_DNxHD
		# Choix du flux audio en sortie pour l'Avid DNxHD
		self.son_sortie_DNxHD = son_sortie_DNxHD
		# Choix du mode (PAL ou NTSC) pour le DV
		self.mode_sortie_DV = mode_sortie_DV
		# Choix de l'aspet (Pas de sélection, 4/3 ou 16/9) pour le DV ou le DVD
		self.aspect_sortie_DV = aspect_sortie_DV
		
		self.son_sortie_General = son_sortie_General
	
	
	def run(self):
		
		for nb_1, fichier_1 in enumerate(self.cheminVideoEntre):
			
			if self.cancel: break
			
			# Option commune à tous les codecs vidéo ----------------------------
			if self.qualite_sortie == 0:
				qual_video = ' -sameq'
			else: 
				qual_video = ' -qscale '+str(self.qualite_sortie)
			# -------------------------------------------------------------------
			
			# Option commune à tous les codecs vidéo ----------------------------
			# Si sélection du DV et du DVD, pas de  
			# choix de la taille de la vidéo en sortie
			if self.codec_sortie in ["DV (.dv)", "DV (.avi)", "DVD (.mpg)"]:
				le_s = ''
				multiplicateur = ''
				'''
				if self.aspect_sortie_DV == "Pas de selection": aspect_dv = ''
				elif self.aspect_sortie_DV == "4/3": aspect_dv = ' -aspect 4:3 '
				elif self.aspect_sortie_DV == "16/9": aspect_dv = ' -aspect 16:9 '
				'''
				if self.aspect_sortie_DV == "0": aspect_dv = '' # Pas de selection
				elif self.aspect_sortie_DV == "1": aspect_dv = ' -aspect 4:3 ' # 4/3
				elif self.aspect_sortie_DV == "2": aspect_dv = ' -aspect 16:9 ' # 16/9
			else: 
				le_s = u' -s '
				multiplicateur = u'x'
				aspect_dv = ''
			# -------------------------------------------------------------------

			# -------------------------------------------------------------------
			# Si l'utilisateur choisit les codecs autres que MPEG 1 et 2, VOB 
			# MPEG4 ou DNxHD, acodec prendra la valeur sélectionnée par 
			# l'utilisateur dans la liste déroulante
			if self.codec_sortie not in ["MPEG2 (.mpg)", "MPEG1 (.mpg)", "VOB (.vob)", "Avid DNxHD (.mov)", "MPEG4 (.mp4)"]:
				if self.son_sortie_General == "0": acodec = ' -acodec copy' # Copie du flux audio
				elif self.son_sortie_General == "1": acodec = ' -ac 2 -acodec pcm_s16le' # Flux audio PCM sur 2 canaux (stereo)
				elif self.son_sortie_General == "2": acodec = ' -an' # Pas de flux audio
			# -------------------------------------------------------------------
			
			if self.codec_sortie == "MOV (.mov)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.mov'
				vcodec = ' -vcodec mpeg4'
				
			elif self.codec_sortie == "MPEG2 (.mpg)":
				fps = ' -r 25'
				extSortie = '.mpg'
				vcodec = ' -vcodec mpeg2video'
				# Le codec audio est spécifique ici
				acodec = ' -acodec mp2 -ac 1 -ar 22050 -ab 64k'
				
			elif self.codec_sortie == "MPEG1 (.mpg)":
				fps = ' -r 25'
				extSortie = '.mpg'
				vcodec = ' -vcodec mpeg1video'
				# Le codec audio est spécifique ici
				acodec = ' -acodec mp2 -ac 1 -ar 22050 -ab 64k'
				
			elif self.codec_sortie == "VOB (.vob)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.vob'
				vcodec = ' '
				# Le codec audio est spécifique ici
				acodec = ' -acodec copy'
				
			elif self.codec_sortie == "MPEG4 (.mp4)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.mp4'
				vcodec = ' -vcodec mpeg4'
				# Le codec audio est spécifique ici
				acodec = ' -acodec copy'
				
			elif self.codec_sortie == "WMV2 (.wmv)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.wmv'
				vcodec = ' -vcodec wmv2'
					
			elif self.codec_sortie == "HFYU (yuv422p) (.avi)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.avi'
				vcodec = ' -vcodec huffyuv -pix_fmt yuv422p'
				
			elif self.codec_sortie == "MSMPEG 4 version 2 (.avi)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.avi'
				vcodec = ' -vcodec msmpeg4v2'
				
			elif self.codec_sortie == "Motion JPEG (.avi)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.avi'
				vcodec = ' -vcodec mjpeg'
				
			elif self.codec_sortie == "FFV1 (FFmpeg) (.avi)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.avi'
				vcodec = ' -vcodec ffv1'

			elif self.codec_sortie == "Avid DNxHD (.mov)":
				# Extension sortie
				extSortie = '.mov'
				# Codec vidéo pour l'Avid DNxHD
				vcodec = ' -vcodec dnxhd'
				# Choix du flux audio pour l'Avid DNxHD
				if self.son_sortie_DNxHD == "0": acodec = ' -acodec copy' # Copie du flux audio
				elif self.son_sortie_DNxHD == "1": acodec = ' -ac 2 -acodec pcm_s16le' # Flux audio PCM sur 2 canaux (stereo)
				elif self.son_sortie_DNxHD == "2": acodec = ' -an' # Pas de flux audio
				# Les valeurs de qualité de la vidéo ne sont pas conservées
				qual_video = ''
				# ------------------------ #
				# Valeurs particulières
				# ------------------------ #
				if self.spec_sortie_DNxHD == '0': # Dimension:1920x1080 Img/sec:29.97 Bitrate:220 Mb/s 
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 29.97 -b 220000k'
				elif self.spec_sortie_DNxHD == '1': # Dimension:1920x1080 Img/sec:29.97 Bitrate:145 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 29.97 -b 145000k'
				elif self.spec_sortie_DNxHD == '2': # Dimension:1920x1080 Img/sec:25 Bitrate:185 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 25 -b 185000k'
				elif self.spec_sortie_DNxHD == '2': # Dimension:1920x1080 Img/sec:25 Bitrate:120 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 25 -b 120000k'
				elif self.spec_sortie_DNxHD == '4': # Dimension:1920x1080 Img/sec:25 Bitrate:36 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 25 -b 36000k'
				elif self.spec_sortie_DNxHD == '5': # Dimension:1920x1080 Img/sec:24 Bitrate:175 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 24 -b 175000k'
				elif self.spec_sortie_DNxHD == '6': # Dimension:1920x1080 Img/sec:24 Bitrate:115 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 24 -b 115000k'
				elif self.spec_sortie_DNxHD == '7': # Dimension:1920x1080 Img/sec:24 Bitrate:36 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 24 -b 36000k'
				elif self.spec_sortie_DNxHD == '8': # Dimension:1920x1080 Img/sec:23.976 Bitrate:175 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 23.976 -b 175000k'
				elif self.spec_sortie_DNxHD == '9': # Dimension:1920x1080 Img/sec:23.976 Bitrate:115 Mb/s"
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 23.976 -b 115000k'
				elif self.spec_sortie_DNxHD == '10': # Dimension:1920x1080 Img/sec:23.976 Bitrate:36 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 23.976 -b 36000k'
				elif self.spec_sortie_DNxHD == '11': # Dimension:1920x1080 Img/sec:29.97 Bitrate:220 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 29.97 -b 220000k'
				elif self.spec_sortie_DNxHD == '12': # Dimension:1920x1080 Img/sec:29.97 Bitrate:145 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 29.97 -b 145000k'
				elif self.spec_sortie_DNxHD == '13': # Dimension:1920x1080 Img/sec:29.97 Bitrate:45 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1920'
					self.reso_hauteur_sortie = '1080'
					# Nbre d'img/sec et bitrate
					fps = ' -r 29.97 -b 45000k'
				elif self.spec_sortie_DNxHD == '14': # Dimension:1280x720 Img/sec:59.94 Bitrate:220 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1280'
					self.reso_hauteur_sortie = '720'
					# Nbre d'img/sec et bitrate
					fps = ' -r 59.94 -b 220000k'
				elif self.spec_sortie_DNxHD == '15': # Dimension:1280x720 Img/sec:59.94 Bitrate:145 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1280'
					self.reso_hauteur_sortie = '720'
					# Nbre d'img/sec et bitrate
					fps = ' -r 59.94 -b 145000k'
				elif self.spec_sortie_DNxHD == '16': # Dimension:1280x720 Img/sec:50 Bitrate:175 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1280'
					self.reso_hauteur_sortie = '720'
					# Nbre d'img/sec et bitrate
					fps = ' -r 50 -b 175000k'
				elif self.spec_sortie_DNxHD == '17': # Dimension:1280x720 Img/sec:50 Bitrate:115 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1280'
					self.reso_hauteur_sortie = '720'
					# Nbre d'img/sec et bitrate
					fps = ' -r 50 -b 115000k'
				elif self.spec_sortie_DNxHD == '18': # Dimension:1280x720 Img/sec:29.97 Bitrate:110 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1280'
					self.reso_hauteur_sortie = '720'
					# Nbre d'img/sec et bitrate
					fps = ' -r 29.97 -b 110000k'
				elif self.spec_sortie_DNxHD == '19': # Dimension:1280x720 Img/sec:29.97 Bitrate:75 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1280'
					self.reso_hauteur_sortie = '720'
					# Nbre d'img/sec et bitrate
					fps = ' -r 29.97 -b 75000k'
				elif self.spec_sortie_DNxHD == '20': # Dimension:1280x720 Img/sec:25 Bitrate:90 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1280'
					self.reso_hauteur_sortie = '720'
					# Nbre d'img/sec et bitrate
					fps = ' -r 25 -b 90000k'
				elif self.spec_sortie_DNxHD == '21': # Dimension:1280x720 Img/sec:25 Bitrate:60 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1280'
					self.reso_hauteur_sortie = '720'
					# Nbre d'img/sec et bitrate
					fps = ' -r 25 -b 60000k'
				elif self.spec_sortie_DNxHD == '22': # Dimension:1280x720 Img/sec:23.976 Bitrate:90 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1280'
					self.reso_hauteur_sortie = '720'
					# Nbre d'img/sec et bitrate
					fps = ' -r 23.976 -b 90000k'
				elif self.spec_sortie_DNxHD == '23': # Dimension:1280x720 Img/sec:23.976 Bitrate:60 Mb/s
					# Taille largeur x hauteur
					self.reso_largeur_sortie = '1280'
					self.reso_hauteur_sortie = '720'
					# Nbre d'img/sec et bitrate
					fps = ' -r 23.976 -b 60000k'
					
			elif self.codec_sortie in ["DV (.dv)", "DV (.avi)"]:
				# Extension sortie
				if self.codec_sortie == "DV (.dv)": extSortie = '.dv'
				elif self.codec_sortie == "DV (.avi)": extSortie = '.avi'
				# Choix du mode (PAL ou NTSC) pour le DV
				if self.mode_sortie_DV == "PAL": vcodec = ' -target pal-dv'
				elif self.mode_sortie_DV == "NTSC": vcodec = ' -target ntsc-dv'
				# Codec audio pour le DV (rien)
				acodec = ''
				# Pas de valeur de qualité de la vidéo
				qual_video = ''
				# Pas de valeurs nbre img/sec
				fps = ''
				# Pas de taille largeur x hauteur
				self.reso_largeur_sortie = ''
				self.reso_hauteur_sortie = ''
				
			elif self.codec_sortie in ["DVD (.mpg)"]:
				# Extension sortie
				extSortie = '.mpg'
				# Choix du mode (PAL ou NTSC) pour le DVD
				if self.mode_sortie_DV == "PAL": vcodec = ' -target pal-dvd'
				elif self.mode_sortie_DV == "NTSC": vcodec = ' -target ntsc-dvd'
				# Codec audio pour le DVD (rien)
				acodec = ''
				# Pas de valeur de qualité de la vidéo
				qual_video = ''
				# Pas de valeurs nbre img/sec
				fps = ''
				# Pas de taille largeur x hauteur
				self.reso_largeur_sortie = ''
				self.reso_hauteur_sortie = ''
				
			# Sélection du drapeau pour savoir si l'utilisateur veut le
			# désentrelacement ou pas et mise place de l'option ... sauf
			# pour le DV et le DVD qui n'utilisent pas le désentrelacement.
			if self.codec_sortie in ["DV (.dv)", "DV (.avi)", "DVD (.mpg)"]:
				self.choixDesent = ''
			else:
				if self.choixDesent == "yes": self.choixDesent = ' -deinterlace'
				elif self.choixDesent == 'no': self.choixDesent = ''
	
			commande = self.cheminFFmpeg+u' -i "'+fichier_1+u'" '+self.choixDesent+le_s+self.reso_largeur_sortie+multiplicateur+self.reso_hauteur_sortie+fps+vcodec+acodec+aspect_dv+qual_video+u' -y  "'+self.enregistrerSortie+u'_'+string.zfill(str(nb_1+1), 5)+extSortie+u'"'
			outputfile = self.enregistrerSortie+'_'+string.zfill(str(nb_1+1), 5)+extSortie

			EkdPrint(u'commande %s' % commande)

			try:
				## Travail avec EkdProcess
				f = EkdProcess(commande.encode(EkdConfig.coding))
				f.communicate()[1]
			except Exception:
				debug(u"Erreur dans le lancement de %s" % commande)
			
			# Ca marche mieux avec pourcent calculé comme ça
			pourcent=((nb_1+1)*100)/len(self.cheminVideoEntre)

			# Emetteur pour la récup des valeurs du pourcentage
			self.emit(SIGNAL("increment(int)"), pourcent)

			# Emetteur pour (essayer) d'afficher chaque fichier chargé
			# (pour info) au bon moment dans la barre de progression
			self.emit(SIGNAL("travail_avec_fichier"), fichier_1, outputfile)
			
			debug ("%d "%(nb_1))
	
		self.emit(SIGNAL("fin process"))
	
	def cancel(self):
		"""SLOT to cancel treatment"""
		self.cancel = True
